<?php
//
//	Module name      :	tbl.php
//	Version          :	1.0
//	Started on       :	April 29, 2005
//	Completed on     :	Maj 6, 2005
//
//	Module type      :	Abstract type
//	Description      :	Functions to create Html tables
//
//	Developed under  :	Php 4.1.2
//	Sw requirements  :	None
//
//	Parameters       :	None
//	Notes - usage    :	Table descriptors are associative arrays that can have following key names and
//						associated types:
//							CellSpacing		: integer					table cell spacing
//							Width			: perc. or integer			table width, referring to parent
//																		dimensions
//							BackColor		: color						table background color
//							BorderWidth		: integer or string			table border width
//							BorderColor		: color						table border color
//							BorderStyle		: string					table border style (none, solid,
//																		double, ...)
//							TitleAlign		: string					title alignment (right, left, center)
//							TitleVAlign		: string					title alignment (top, middle,
//																		bottom, ...)
//							TitlePadding	: string					title cell padding (0px, 2px, ...)
//							TitleBackColor	: color						title cell background color
//							TitleColor		: color						title text color
//							TitleFontFamily	: string					title text font (Arial, Times,
//																		Courier)
//							TitleFontSize	: string					title text size (8px, 10px, ...)
//							TitleFontWeight	: string					title text style (normal, bold, 100,
//																		200, ...)
//							TitleBorderWidth: integer or string			title border width
//							TitleBorderColor: color						title border color
//							TitleBorderStyle: string					title border style (none, solid,
//																		double, ...)
//							Columns			: integer					table columns number
//							ColAlign		: array[0..n] of string		columns alignment (right, left,
//																		center)
//							ColWidth		: array[0..n] of perc./int	columns width
//							ColVAlign		: array[0..n] of string		columns alignment (top, middle,
//																		bottom, ...)
//							ColPadding		: array[0..n] of string		columns cell padding (0px, 2px, ...)
//							ColBackColor	: array[0..n] of color		columns cell background color
//							ColColor		: array[0..n] of color		columns text color
//							ColFontFamily	: array[0..n] of string		columns text font (Arial, Times,
//																		Courier)
//							ColFontSize		: array[0..n] of string		columns text size (8px, 10px, ...)
//							ColFontWeight	: array[0..n] of string		columns text style (normal, bold, 100,
//																		200, ...)
//							ColBorderWidth	: array[0..n] of int/string	columns border width
//							ColBorderColor	: array[0..n] of color		columns border color
//							ColBorderStyle	: array[0..n] of string		columns border style (none, solid,
//																		double, ...)
//						In array[0..n] type, if n < (Columns - 1) then the n-th value is used for remaining
//						columns; if the array is empty the default value is used.
//						Alternative attributes sets can be used for interlacing rows; their descriptors are
//						associative arrays with only key names beginning with "Col" (of course "ColWidth"
//						cannot work correctly after the first row).
//


//--- Include files --------------------------------------------------------------------------------



require_once("indent.php");



//--- Public constants -----------------------------------------------------------------------------



define("TBL_MODULE_NAME", "tbl.php 1.0; on Maj 6, 2005 by J.Hubb.");

// Error codes
define("TBL_NO_ERROR",		0);
define("TBL_DESC_ERROR",	1);
define("TBL_NOT_IMP_YET",	-1);
define("TBL_LAST_ERROR",	1);



//--- Public variables -----------------------------------------------------------------------------



//--- Private constants ----------------------------------------------------------------------------



//--- Private variables ----------------------------------------------------------------------------



$_TBL_error_num = TBL_NO_ERROR;
$_TBL_error_msg = array(
	TBL_NO_ERROR	=> "No error",
	TBL_DESC_ERROR	=> "Invalid table descriptor",
	TBL_NOT_IMP_YET	=> "Feature not implemented yet"
	);



//--- Public functions code ------------------------------------------------------------------------



function TBLCell(&$desc, $elem = "&nbsp;")
// Returns, for the table with descriptor $desc, the printable string for a single cell containing $elem,
// handling opening and closing of the relative row.
{
	global $_TBL_error_num;

	$_TBL_error_num = TBL_NOT_IMP_YET;
	return "";
}



function TBLDone(&$desc)
// Returns, for the table with the descriptor $desc, the string for the closure.
{
	global $_TBL_error_num;

	if (array_key_exists("_Init", $desc) && ($desc["_Init"] === TRUE))
	{
		unset($desc["_Init"], $desc["_TagsTable"], $desc["_TagsTitle"], $desc["_TagsCol"],
				$desc["_TagsColCurr"]);
		$result = ILess() . "</table>\n";
		$_TBL_error_num = TBL_NO_ERROR;
	}
	else
	{
		$result = "";
		$_TBL_error_num = TBL_DESC_ERROR;
	}
	return $result;
}



function TBLEmpty(&$desc, $elem = "")
// Returns, for the table with descriptor $desc, the printable string for a empty row (without cell
// background); the optional $elem is spanned over all columns.
{
	global $_TBL_error_num;

	if (@$desc["_Init"] !== TRUE)
	{
		$_TBL_error_num = TBL_DESC_ERROR;
		return FALSE;
	}
	$_TBL_error_num = TBL_NO_ERROR;
	return IInd() . "<tr>\n" . IMore() . @"<td colspan=$desc[Columns]>\n" . IMore() .
			"{$elem}\n" . ILess() . "</td>\n" . ILess() . "</tr>\n";
}



function TBLErrorMsg()
// Returns the error message of the last module function called.
{
	global $_TBL_error_msg, $_TBL_error_num;

	return $_TBL_error_msg[$_TBL_error_num];
}



function TBLInit(&$desc, $attr_set = array())
// Initializes a table descripted in the associative array $desc; the array can have the keys names listed
// in Notes. If keys are not found, then $desc is modified inserting these keys with default values. All tags
// not recognized are ignored.
// The optional parameter $attr_set is an array with elements describing alternative attributes for
// interlacing rows; these elements are associative arrays too; keys names can be all those tags listed in
// Notes that begin with "Col".
// Returns table header; sets error code always to TBL_NO_ERROR.
{
	global $_TBL_error_num;
	static $def = array(							// $def and $def_col must not have common keys.
			"CellSpacing"		=> "0",
			"Width"				=> "100%",
			"BackColor"			=> "",
			"BorderWidth"		=> "1",
			"BorderColor"		=> "black",
			"BorderStyle"		=> "solid",
			"TitleAlign"		=> "center",
			"TitleVAlign"		=> "middle",
			"TitlePadding"		=> "0px",
			"TitleBackColor"	=> "",
			"TitleColor"		=> "",
			"TitleFontFamily"	=> "Arial,Helvetica,sans-serif",
			"TitleFontSize"		=> "",
			"TitleFontWeight"	=> "normal",
			"TitleBorderWidth"	=> "1",
			"TitleBorderColor"	=> "",
			"TitleBorderStyle"	=> "solid",
			"Columns"			=> "1"
			);
	static $def_col = array(
			"ColAlign"			=> array(""),
			"ColWidth"			=> array(""),
			"ColVAlign"			=> array(""),
			"ColPadding"		=> array("0px"),
			"ColBackColor"		=> array(""),
			"ColColor"			=> array(""),
			"ColFontFamily"		=> array("Arial,Helvetica,sans-serif"),
			"ColFontSize"		=> array(""),
			"ColFontWeight"		=> array("normal"),
			"ColBorderWidth"	=> array("1"),
			"ColBorderColor"	=> array(""),
			"ColBorderStyle"	=> array("solid")
			);

	$desc = array_merge($def, $def_col, $desc, array("_Init" => TRUE));	// Overwrite default values with
	$col_num = (int) $desc["Columns"];									// user's ones.
	$col_num = ($col_num > 0 ? $col_num : 1);
	//
	// Prepare format tags for <table>.
	//
	if ($desc["CellSpacing"] == "")
		$desc["CellSpacing"] = "0";
	if ($desc["Width"] == "")
		$desc["Width"] = "100%";
	$desc["_TagsTable"] = "style='" .
			($desc["BackColor"]		? "background-color:$desc[BackColor];"	: "") .
			($desc["BorderWidth"]	? "border-width:$desc[BorderWidth];"	: "") .
			($desc["BorderColor"]	? "border-color:$desc[BorderColor];"	: "") .
			($desc["BorderStyle"]	? "border-style:$desc[BorderStyle];'"	: "'");
	//
	// Prepare format tags for title sections.
	//
	$desc["_TagsTitle"] =
			($desc["TitleAlign"]		? "align=$desc[TitleAlign] "				: "") .
			"style='" .
			($desc["TitleVAlign"]		? "vertical-align:$desc[TitleVAlign];"	: "") .
			($desc["TitlePadding"]		? "padding:$desc[TitlePadding];"	: "") .
			($desc["TitleBackColor"]	? "background-color:$desc[TitleBackColor];"	: "") .
			($desc["TitleColor"]		? "color:$desc[TitleColor];"				: "") .
			($desc["TitleFontFamily"]	? "font-family:$desc[TitleFontFamily];"		: "") .
			($desc["TitleFontSize"]		? "font-size:$desc[TitleFontSize];"			: "") .
			($desc["TitleFontWeight"]	? "font-weight:$desc[TitleFontWeight];"		: "") .
			($desc["TitleBorderWidth"]	? "border-width:$desc[TitleBorderWidth];"	: "") .
			($desc["TitleBorderColor"]	? "border-color:$desc[TitleBorderColor];"	: "") .
			($desc["TitleBorderStyle"]	? "border-style:$desc[TitleBorderStyle];'"	: "'");
	//
	// Prepare format tags for row sections.
	//
	$desc["_TagsCol"] = array();
	foreach ($def_col as $key => $val)		// Check that arrays are not empty and pad them till "Columns".
		if (is_array($val))
		{
			settype($desc[$key], "array");
			$desc[$key] = array_pad($desc[$key], $col_num, array_pop($desc[$key]));
		}
	for ($n = 0; $n < $col_num; $n++)
		$desc["_TagsCol"][0][$n] = _TBLBuildTagsCol($desc, $n);
	$desc["_TagsColCurr"] = 0;
	for ($p = 0; $p < count($attr_set); $p++)
	{
		$tmp_attr = array_merge($desc, $attr_set[$p]);
		foreach ($def_col as $key => $val)
			if (is_array($val))
			{
				settype($tmp_attr[$key], "array");
				$tmp_attr[$key] = array_pad($tmp_attr[$key], $col_num, array_pop($tmp_attr[$key]));
			}
		for ($n = 0; $n < $col_num; $n++)
			$desc["_TagsCol"][$p + 1][$n] = _TBLBuildTagsCol($tmp_attr, $n);
	}
	$result = IInd() . "<table cellspacing='$desc[CellSpacing]' width='$desc[Width]' $desc[_TagsTable]>\n";
	IMore();
	$_TBL_error_num = TBL_NO_ERROR;
	return $result;
}



function TBLNextSet(&$desc, $n)
// Sets the next alternative attribute set to be used for interlacing rows; returns FALSE on errors or an
// empty string (so, it can be used during table printing). $n can be 0 to refer to attribute set provided
// with the original descriptor; greater values refer to alternate sets. $n is always normalized to valid
// values.
{
	global $_TBL_error_num;

	if (@$desc["_Init"] !== TRUE)
	{
		$_TBL_error_num = TBL_DESC_ERROR;
		return FALSE;
	}
	$_TBL_error_num = TBL_NO_ERROR;
	$desc["_TagsColCurr"] = ((integer) $n) % count(@$desc["_TagsCol"]);
	return "";
}



function TBLResult()
// Returns the error code of the last module function called.
{
	global $_TBL_error_num;

	return $_TBL_error_num;
}



function TBLRow(&$desc)
// Returns, for the table with descriptor $desc, the printable string for a row; row cells can be passed
// as variable arguments list, after $desc. If the first one after $desc is an array, then its elements will
// be used as cell data; otherwise every parameter found will be put into cells. If values are not enough,
// remaining are left empty (with a "&nbsp;"); exceeding values are ignored.
{
	global $_TBL_error_num;

	if (@$desc["_Init"] !== TRUE)
	{
		$_TBL_error_num = TBL_DESC_ERROR;
		return FALSE;
	}
	$_TBL_error_num = TBL_NO_ERROR;
	$curr_set = (is_integer(@$desc["_TagsColCurr"]) ? $desc["_TagsColCurr"] : 0);
	$desc["_TagsColCurr"] = ($curr_set + 1) % count(@$desc["_TagsCol"]);
	if ((func_num_args() > 1) && (is_array(func_get_arg(1))))
		$p = array_pad(array_values(func_get_arg(1)), @$desc["Columns"], "&nbsp;");
	else
	{
		$p = array();
		for ($n = 1; $n <= @$desc["Columns"]; $n++)
			$p[$n - 1] = ($n < func_num_args() ? func_get_arg($n) : "&nbsp;");
	}
	$result = IInd() . "<tr>\n";
	IMore();
	for ($n = 0; $n < @$desc["Columns"]; $n++)
		$result .= IInd() . @"<td {$desc['_TagsCol'][$curr_set][$n]}>\n" . IMore() . "{$p[$n]}\n" .
				ILess() . "</td>\n";
	return $result . ILess() . "</tr>\n";
}



function TBLTitle(&$desc, $title = "")
// Returns, for the table with descriptor $desc, the printable string for a title row; the optional $title
// is spanned over all columns.
{
	global $_TBL_error_num;

	if (@$desc["_Init"] !== TRUE)
	{
		$_TBL_error_num = TBL_DESC_ERROR;
		return FALSE;
	}
	$_TBL_error_num = TBL_NO_ERROR;
	return IInd() . "<tr>\n" . IMore() . "<td colspan=$desc[Columns] $desc[_TagsTitle]>\n" . IMore() .
			"{$title}\n" . ILess() . "</td>\n" . ILess() . "</tr>\n";
}



//--- Private functions code -----------------------------------------------------------------------



function _TBLBuildTagsCol(&$attr, $i)
// Returns the format tags string for column $i, using descriptor $attr; $attr is passed by reference
// to optimize speed but it is not modified.
{
	return  ($attr["ColAlign"][$i]			? "align={$attr['ColAlign'][$i]} "					: "") .
			($attr["ColWidth"][$i]			? "width={$attr['ColWidth'][$i]} "					: "") .
			"style='" .
			($attr["ColVAlign"][$i]			? "vertical-align:{$attr['ColVAlign'][$i]};"		: "") .
			($attr["ColPadding"][$i]		? "padding:{$attr['ColPadding'][$i]};"				: "") .
			($attr["ColBackColor"][$i]		? "background-color:{$attr['ColBackColor'][$i]};"	: "") .
			($attr["ColColor"][$i]			? "color:{$attr['ColColor'][$i]};"					: "") .
			($attr["ColFontFamily"][$i]		? "font-family:{$attr['ColFontFamily'][$i]};"		: "") .
			($attr["ColFontSize"][$i]		? "font-size:{$attr['ColFontSize'][$i]};"			: "") .
			($attr["ColFontWeight"][$i]		? "font-weight:{$attr['ColFontWeight'][$i]};"		: "") .
			($attr["ColBorderWidth"][$i]	? "border-width:{$attr['ColBorderWidth'][$i]};"		: "") .
			($attr["ColBorderColor"][$i]	? "border-color:{$attr['ColBorderColor'][$i]};"		: "") .
			($attr["ColBorderStyle"][$i]	? "border-style:{$attr['ColBorderStyle'][$i]};'"	: "'");
}



//--- Initialization code --------------------------------------------------------------------------



?>
